/*
 * main.h
 *
 * Header file for OpenIVR
 *
 * A H.323 IVR application.
 *
 * Copyright (C) 2002 Equivalence Pty. Ltd.
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 *
 * The Original Code is Portable Windows Library.
 *
 * The Initial Developer of the Original Code is Equivalence Pty. Ltd.
 *
 * Contributor(s): ______________________________________.
 *
 * $Log: main.h,v $
 * Revision 1.1  2002/08/14 07:09:03  craigs
 * Initial version
 *
 * Revision 1.8  2002/07/29 13:06:55  csoutheren
 * Updated to latest sources
 *
 *
 */

#ifndef _OpenIVR_MAIN_H
#define _OpenIVR_MAIN_H

#include <ptclib/shttpsvc.h>
#include <ptclib/ptts.h>

#include <opalvxml.h>
#include <h323.h>
#include <h323pdu.h>
#include <h245.h>

class MyH323EndPoint;

class OpenIVR : public PSecureHTTPServiceProcess
{
  PCLASSINFO(OpenIVR, PSecureHTTPServiceProcess)

  public:
    OpenIVR();

    // overrides from PHTTPServiceProcess
    void Main();
    BOOL OnStart();
    void OnStop();
    void OnConfigChanged();
    void AddUnregisteredText(PHTML & html);
    BOOL Initialise(const char * initMsg);
    void OnControl() { }

    // new functions
    PStringArray GetCodecsToRemove() const  { return codecsToRemove; }
    PStringArray GetCodecsToReorder() const { return codecsToReorder; }
    PString GetVXMLDocument() const         { return vxmlDocument; }
    BOOL IsG7231Allowed() const             { return allowG7231; }

    void OutputLogLine(const PTime & time, 
                       const PString & addr,
                       const PString & dn,
                       PINDEX duration,
                       const PString & codec);

  protected:
    PSyncPointAck exitMain;
    PMutex reloadMutex;
    PMutex logMutex;
    BOOL allowG7231;

    MyH323EndPoint * endpoint;

    PStringArray codecsToRemove;
    PStringArray codecsToReorder;

    PFilePath callLogFn;
    PFilePath vxmlDocument;
};

class MyH323EndPoint : public H323EndPoint
{
  PCLASSINFO(MyH323EndPoint, H323EndPoint);

  public:
    MyH323EndPoint(OpenIVR & _app);

    // overrides from H323EndPoint
    virtual H323Connection * CreateConnection(unsigned callReference);

    // new functions
    BOOL Initialise(PConfig & cfg, PConfigPage * rsrc);

  protected:
    OpenIVR & app;
};

//////////////////////////////////////////////////////////////////

class MyH323Connection;

class IVRVXMLSession : public OpalVXMLSession
{
  PCLASSINFO(IVRVXMLSession, OpalVXMLSession);
  public:
    IVRVXMLSession(OpenIVR & _app, MyH323Connection * _conn);

  protected:
    OpenIVR & app;
};

//////////////////////////////////////////////////////////////////

class MyH323Connection : public H323Connection
{
  PCLASSINFO(MyH323Connection, H323Connection);

  public:
    MyH323Connection(MyH323EndPoint &, unsigned, OpenIVR & app);
    ~MyH323Connection();

    // overrides from H323Connection
    BOOL OpenAudioChannel(BOOL, unsigned, H323AudioCodec & codec);
    AnswerCallResponse OnAnswerCall(const PString &, const H323SignalPDU &, H323SignalPDU &);
    BOOL OnReceivedSignalSetup(const H323SignalPDU & setupPDU);
    void OnUserInputString(const PString & value);
    void OnSetLocalCapabilities();

  protected:
    const MyH323EndPoint & ep;

    OpenIVR & app;
    PMutex  connMutex;
    PINDEX audioChannelsOpen;

    PString e164Number;
    PString srcAddress;

    IVRVXMLSession vxml;

    PTime callStartTime;
    PString codecName;
};

//////////////////////////////////////////////////////////////////

#endif


// End of File ///////////////////////////////////////////////////////////////
